Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Mssql.RowSourceA row of data accessible by column name with helper functions for conversions.
to_alist t converts t to a list of (colname, value) pairs
bignum colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to a bignum or wasn't in the query.
bignum_exn colname similar to bignum but asserts that the column named colname exists and the data is not null.
bool colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to a bool or wasn't in the query.
bool_exn colname similar to bool but asserts that the column named colname exists and the data is not null.
float colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to a float or wasn't in the query.
float_exn colname similar to float but asserts that the column named colname exists and the data is not null.
int colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to an int or wasn't in the query.
int_exn colname similar to int but asserts that the column named colname exists and the data is not null.
int32 colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to an int32 or wasn't in the query.
int32_exn colname similar to int32 but asserts that the column named colname exists and the data is not null.
int64 colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to an int64 or wasn't in the query.
int64_exn colname similar to int64 but asserts that the column named colname exists and the data is not null.
str colname returns the value of the given column, or None if the column is null. Throws an exception if the column wasn't in the query.
str_exn colname similar to str but asserts that the column named colname exists and the data is not null.
date colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to a bool or wasn't in the query.
date_exn colname similar to date but asserts that the column named colname exists and the data is not null.
datetime colname returns the value of the given column, or None if the column is null. Throws an exception if the column can't be converted to a bool or wasn't in the query.